﻿Public Class mainForm
    'Define an array with booleans for each key
    Dim Keys(255) As Boolean

    'Define a structure with all relevant axis components
    Public Structure axis_componets
        Public velocity As Integer
        Public velocity_MAX As Integer
        Public acceleration As Integer
        Public deceleration As Integer
    End Structure

    'Initialize axis components for X and Y
    Public X As New axis_componets
    Public Y As New axis_componets

    'Establish controller handle
    WithEvents g As New Galil.Galil

    'Define Keycodes for directional keys
    Enum KeyCodes
        left = 37
        right = 39
        up = 38
        down = 40
    End Enum

    Private Sub mainForm_Load() Handles MyBase.Load
        'Connect the controller
        'g.address = "10.0.5.72"
        g.address = ""
        Me.Text = g.connection

        'Initialize Motion Parameters for X and Y
        X.velocity = 0
        Y.velocity = 0
        X.velocity_MAX = 10000
        Y.velocity_MAX = 10000
        X.acceleration = 400
        Y.acceleration = 400
        X.deceleration = 800
        Y.deceleration = 800

        'Rotate arrow icons to correct orientation
        down_arrow.Image.RotateFlip(RotateFlipType.Rotate180FlipNone)
        right_arrow.Image.RotateFlip(RotateFlipType.Rotate90FlipNone)
        left_arrow.Image.RotateFlip(RotateFlipType.Rotate270FlipNone)

        'Start The Timer Up
        Timer1.Start()
    End Sub

    'This allows arrow key presses when focus on stop_jog button
    'Intercepts default move to other button
    Private Sub stop_jog_PreviewKeyDown(ByVal sender As System.Object, ByVal e As System.Windows.Forms.PreviewKeyDownEventArgs) Handles stop_jog.PreviewKeyDown
        Select Case (e.KeyCode)
            Case KeyCodes.left, KeyCodes.right, KeyCodes.up, KeyCodes.down
                e.IsInputKey = True
        End Select
    End Sub
    'This allows arrow key presses when focus on begin_jog button
    'Intercepts default move to other button
    Private Sub begin_jog_PreviewKeyDown(ByVal sender As System.Object, ByVal e As System.Windows.Forms.PreviewKeyDownEventArgs) Handles begin_jog.PreviewKeyDown
        Select Case (e.KeyCode)
            Case KeyCodes.left, KeyCodes.right, KeyCodes.up, KeyCodes.down
                e.IsInputKey = True
        End Select
    End Sub

    'If key is pressed then set the corresponding boolean of the index in the key array to true
    Private Sub mainForm_KeyDown(ByVal sender As System.Object, ByVal e As System.Windows.Forms.KeyEventArgs) Handles MyBase.KeyDown
        Keys(e.KeyValue) = True
    End Sub
    Private Sub mainForm_KeyUp(ByVal sender As System.Object, ByVal e As System.Windows.Forms.KeyEventArgs) Handles MyBase.KeyUp
        Keys(e.KeyValue) = False
    End Sub

    'Begin Jog Button Action
    Private Sub begin_jog_Click() Handles begin_jog.Click
        'Connect server and start motion
        g.command("SH XY")
        g.command("BG XY")
    End Sub

    'Stop Jog Button Action
    Private Sub stop_jog_Click() Handles stop_jog.Click
        g.command("ST XY")
    End Sub

    'If form arrow image is clicked on then set the index in the key array to true
    Private Sub up_arrow_MouseDown() Handles up_arrow.MouseDown
        Keys(KeyCodes.up) = True
    End Sub
    Private Sub down_arrow_MouseDown() Handles down_arrow.MouseDown
        Keys(KeyCodes.down) = True
    End Sub
    Private Sub left_arrow_MouseDown() Handles left_arrow.MouseDown
        Keys(KeyCodes.left) = True
    End Sub
    Private Sub right_arrow_MouseDown() Handles right_arrow.MouseDown
        Keys(KeyCodes.right) = True
    End Sub

    'If form arrow image click is released then set the index in the key array to false
    Private Sub up_arrow_MouseUp() Handles up_arrow.MouseUp
        Keys(KeyCodes.up) = False
    End Sub
    Private Sub down_arrow_MouseUp() Handles down_arrow.MouseUp
        Keys(KeyCodes.down) = False
    End Sub
    Private Sub left_arrow_MouseUp() Handles left_arrow.MouseUp
        Keys(KeyCodes.left) = False
    End Sub
    Private Sub right_arrow_MouseUp() Handles right_arrow.MouseUp
        Keys(KeyCodes.right) = False
    End Sub

    'Every timer tick of the clock update the velocity vector and send motion command
    Private Sub Timer1_Tick() Handles Timer1.Tick
        'My.Application.Log.WriteEntry(X.velocity.ToString + " " + X.velocity_MAX.ToString)

        'Check the Limits and process command X
        If (System.Math.Abs(X.velocity) < X.velocity_MAX) Then
            'If left or right direction then accelerate the Y axis
            If Keys(KeyCodes.right) Then 'Right
                X.velocity += X.acceleration
            End If
            If Keys(KeyCodes.left) Then 'Left
                X.velocity -= X.acceleration
            End If
        End If

        'Check the Limits and process command Y
        If (System.Math.Abs(Y.velocity) < Y.velocity_MAX) Then
            'If up or down direction then accelerate the Y axis
            If Keys(KeyCodes.up) Then 'UP
                Y.velocity += Y.acceleration
            End If
            If Keys(KeyCodes.down) Then 'Down
                Y.velocity -= Y.acceleration
            End If
        End If

        'If there is no directional input then decelerate
        If (Keys(KeyCodes.left) + Keys(KeyCodes.right) = 0) Then
            'Reduce speeds to zero if under decelleration threshold
            'necessary when deceleration is not a multiple of acceleration
            If (System.Math.Abs(X.velocity) < X.deceleration) Then
                X.velocity = 0
            End If

            'Decelerate the X
            If X.velocity > 0 Then
                X.velocity -= X.deceleration
            ElseIf X.velocity < 0 Then
                X.velocity += X.deceleration
            End If
        End If
        If (Keys(KeyCodes.up) + Keys(KeyCodes.down) = 0) Then
            'Reduce speeds to zero if under decelleration threshold
            'necessary when deceleration is not a multiple of acceleration
            If (System.Math.Abs(Y.velocity) < Y.deceleration) Then
                Y.velocity = 0
            End If

            'Decelerate the Y
            If Y.velocity > 0 Then
                Y.velocity -= Y.deceleration
            ElseIf Y.velocity < 0 Then
                Y.velocity += Y.deceleration
            End If
        End If

        'Update velocity output displays
        X_velocity_display.Text = X.velocity.ToString()
        Y_velocity_display.Text = Y.velocity.ToString()

        'Send the velocity commands
        g.command("JG " + X.velocity.ToString + "," + Y.velocity.ToString)
        'My.Application.Log.WriteEntry("JG " + X.velocity.ToString + "," + Y_velocity.ToString)
    End Sub
End Class

